<?php
/**
 * 2024 Spedire Web
 *
 * Last update 23/11/2022 - 02:49:53
 * Last update 04/05/2024 - 16:12:38
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@spedireweb.com so we can send you a copy immediately.
 *
 *
 *  @author    Puntobox Italia SRL <info@spedireweb.com>
 *  @copyright Puntobox Italia SRL
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  www.spedireweb.com
 */
if (!defined("_PS_VERSION_")) {
    exit();
}
class SpedireWeb extends Module
{
    protected $config_form = false;
    public function __construct()
    {
        $this->name = "Spedire Web";
        $this->tab = "shipping_logistics";
        $this->version = "1.0.5";
        $this->author = "Puntobox Italia SRL";
        $this->author_uri = 'https://www.spedireweb.com';
        $this->need_instance = 0;
        $this->api_test_data = [];
        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l("Spedire Web");
        $this->description = $this->l(
            "Synchronize your orders with your Spedire Web Account."
        );
        $this->ps_versions_compliancy = ["min" => "1.6", "max" => _PS_VERSION_];
    }
    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue("SPEDIREWEB_AUTH_TOKEN", "");
        return parent::install() &&
            $this->registerHook("header") &&
            $this->registerHook("backOfficeHeader") &&
            $this->registerHook("actionOrderStatusUpdate") &&
            $this->registerHook("actionOrderStatusPostUpdate") &&
            $this->registerHook("actionValidateOrder") &&
            $this->registerHook('displayAdminOrderTabShip');
    }
    public function uninstall()
    {
        Configuration::deleteByName("SPEDIREWEB_AUTH_TOKEN");
        return parent::uninstall();
    }
    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
		//$api_response = ["code" => 0, "status" =>"", "details" => ""];
		$api_response = $this->makeCurlPostRequest(
            $this->api_test_data,
            Configuration::get("SPEDIREWEB_AUTH_TOKEN")
        );
		if (((bool) Tools::isSubmit("submitSpedireWebModule")) == true) {
            $api_response = $this->postProcess();
        }
        $form = $this->renderForm();
        $this->context->smarty->assign([
            "SPEDIREWEB_AUTH_TOKEN" => Configuration::get("SPEDIREWEB_AUTH_TOKEN"),
            "form" => $form,
            "api_response" => $api_response,
        ]);        
        $output = $this->context->smarty->fetch(
            $this->local_path . "views/templates/admin/configure.tpl"
        );
        return $output;
    }
    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get(
            "PS_BO_ALLOW_EMPLOYEE_FORM_LANG",
            0
        );
        $helper->identifier = $this->identifier;
        $helper->submit_action = "submitSpedireWebModule";
        $helper->currentIndex =
            $this->context->link->getAdminLink("AdminModules", false) .
            "&configure=" .
            $this->name .
            "&tab_module=" .
            $this->tab .
            "&module_name=" .
            $this->name;
        $helper->token = Tools::getAdminTokenLite("AdminModules");
        $helper->tpl_vars = [
            "fields_value" => $this->getConfigFormValues() /* Add values for your inputs */,
            "languages" => $this->context->controller->getLanguages(),
            "id_language" => $this->context->language->id,
        ];
        return $helper->generateForm([$this->getConfigForm()]);
    }
    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return [
            "form" => [
                "legend" => [
                    "title" => $this->l("Connect with Spedire Web"),
                    "icon" => "icon-cogs",
                ],
                "input" => [
                    [
                        "col" => 3,
                        "type" => "text",
                        "prefix" => '<i class="icon icon-key"></i>',
                        "desc" => $this->l("Enter your Spedire Web Auth Token."),
                        "name" => "SPEDIREWEB_AUTH_TOKEN",
                        "label" => $this->l("Auth Token"),
                    ],
                ],
                "submit" => ["title" => $this->l("Save")],
            ],
        ];
    }
    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return [
            "SPEDIREWEB_AUTH_TOKEN" => Configuration::get(
                "SPEDIREWEB_AUTH_TOKEN",
                ""
            ),
        ];
    }
    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $SPEDIREWEB_AUTH_TOKEN = Tools::getValue("SPEDIREWEB_AUTH_TOKEN");

        $response = $this->makeCurlPostRequest(
            $this->api_test_data,
            $SPEDIREWEB_AUTH_TOKEN
        );

        Configuration::updateValue("SPEDIREWEB_AUTH_TOKEN", $SPEDIREWEB_AUTH_TOKEN);
        return $response;
    }
    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue("module_name") == $this->name) {
            $this->context->controller->addJS(
                $this->_path . "views/js/back.js"
            );
            $this->context->controller->addCSS(
                $this->_path . "views/css/back.css"
            );
        }
    }
    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . "/views/js/front.js");
        $this->context->controller->addCSS(
            $this->_path . "/views/css/front.css"
        );
    }
    //Called after the status of an order changes.
    public function hookActionOrderStatusPostUpdate($params)
    {
        /* Place your code here. */
        $order = new Order($params["id_order"]);
        $customer = new Customer($order->id_customer);
		$products = $order->getProducts();
        $shipping_address = new Address($order->id_address_delivery);
        $invoice_address = new Address($order->id_address_invoice);
        $country_obj = new Country($shipping_address->id_country);
        $state_obj = new State($shipping_address->id_state);
        $receiver = [
            "first_name" => $shipping_address->firstname,
            "last_name" => $shipping_address->lastname,
            "at" => $shipping_address->company, //comapnyname
            "address" => $shipping_address->address1,
            "address_2" => $shipping_address->address2,
            "zip_code" => $shipping_address->postcode,
            "city" => $shipping_address->city,
            "country" => $shipping_address->country,
            "province" => $state_obj->name,
            "phone" => $shipping_address->phone,
            "mobile" =>
                $shipping_address->phone_mobile != ""
                    ? $shipping_address->phone_mobile
                    : "9911221212",
            "email" => $customer->email,
            "notes" => $shipping_address->other,
        ];
        /*$customer_id =
            $customer->firstname .
            " " .
            $customer->lastname .
            " [" .
            $order->reference .
            "]" .
		*/
		//$customer_id = $customer->lastname . " " . $customer->firstname;
		$customer->email;
		$loop_product = array();	
		foreach($products AS $product){
			$id_product = $product["id_product"];
			$weight = $product["weight"];
			$height = $product["height"];
			$width = $product["width"];
			$length = $product["depth"];
			$product_quantity = $product["product_quantity"];
			$loop_product[] = array("id"=>$id_product,"weight"=>$weight,"height"=>$height,"width"=>$width,"length"=>$length,"quantity"=>$product_quantity);
		}
		$details_produts = array("products" => $loop_product);
        $data = [
            "platform_id" => 1,
            "order_id" => $order->reference,
            //"customer_name" => $customer_name,
            "products" => $loop_product,			
            "receiver" => $receiver,
            "status" => $order->current_state,
            //"text" => json_encode($products, true),
        ];
        $this->makeCurlPostRequest(
            $data,
            Configuration::get("SPEDIREWEB_AUTH_TOKEN")
        );
    }
    public function makeCurlPostRequest($data, $token)
    {
        $data_string = $data;

        $data_string = json_encode($data_string);
        $url = "https://api.spedireweb.it/1.0/import/";
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_USERAGENT, "SpedireWeb");
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            "Content-Type: application/json",
            "Authorization: " . $token,
        ]);
        $content = curl_exec($ch);
        $response_obj = (array) json_decode($content);
        curl_close($ch);
        return $response_obj;
        if (isset($response_obj["status"])) {
            if ($response_obj["status"] == "error") {
                echo $response_obj["details"];
                exit();
            }
        }
    }
    public function hookActionValidateOrder($params)
    {
        /* Place your code here. */
        $id_customer = (int) $params["customer"]->id;
        $id_order = (int) $params["order"]->id;
        $customer = new Customer($id_customer);
        $order = new Order($id_order);
        $shipping_address = new Address($order->id_address_delivery);
        $invoice_address = new Address($order->id_address_invoice);
        $country_obj = new Country($shipping_address->id_country);
        $state_obj = new State($shipping_address->id_state);
        $receiver = [
            "first_name" => $shipping_address->firstname,
            "last_name" => $shipping_address->lastname,
            "at" => $shipping_address->company, //comapnyname
            "address" => $shipping_address->address1,
            "address_2" => $shipping_address->address2,
            "zip_code" => $shipping_address->postcode,
            "city" => $shipping_address->city,
            "country" => $shipping_address->country,
            "province" => $state_obj->name,
            "phone" => $shipping_address->phone,
            "mobile" =>
                $shipping_address->phone_mobile != ""
                    ? $shipping_address->phone_mobile
                    : "",
            "notes" => $shipping_address->other,
        ];
        $customer_id =
            $customer->firstname .
            " " .
            $customer->lastname .
            " [" .
            $order->reference .
            "]" .
            $customer->email;
        $data = [
            "platform_id" => 1,
            "order_id" => $order->reference,
            "customer_id" => $customer_id,
            "receiver" => $receiver,
            "status" => $order->current_state,
        ];
        $this->makeCurlPostRequest(
            $data,
            Configuration::get("SPEDIREWEB_AUTH_TOKEN")
        );
    }
}
